<?php

/**
 * 验证码类
 * @author John <fsyzxz@163.com>
 */
class Verify
{
    private $width = 200;       //验证码的宽度
    private $height = 80;       //验证码的高度
    private $type = 1;          //验证码的类型
    private $length = 4;        //验证码的长度
    private $code;              //验证码
    private $img;               //图像的资源
    private $seKey = 'John';    //密钥
    private $bgRand = true;     //随机背景图片
    
    /**
     * 构造函数
     * @param type $width 验证码的宽度
     * @param type $height 验证码的高度
     * @param type $length 验证码的长度
     * @param type $type 验证码的类型
     */
    public function __construct($width = 200, $height = 80, $length = 4, $type = 1)
    {
        $this->width  = !empty($width)  ? $width  : $this->width;
        $this->height = !empty($height) ? $height : $this->height;
        $this->length = !empty($length) ? $length : $this->length;
        $this->type   = !empty($type)   ? $type   : $this->type;
    }
    
    /**
     * 设置属性值
     * @param type $name 属性名
     * @param type $value 属性值
     */
    public function __set($name, $value)
    {
        if (isset($name)) {
            $this->$name = $value;
        }
    }
    
    /**
     * 获取属性值
     * @param type $name 属性名
     * @return type 返回属性值
     */
    public function __get($name) {
        return $this->$name;
    }
    
    /**
     * 校验验证码
     * @param type $code 表单提供的验证码
     * @return boolean
     */
    public function check($code){
        if (!isset($_SESSION)) {session_start();}
        if ($this->encodeVerify(strtolower($code)) === $_SESSION['code']){
            return true;
        }else{
            return false;
        }
    }
    
    //输出验证码
    public function verify()
    {
        $this->code = $this->createVerify();
        //创建背景
        $this->createBackground();
        //文字显示
        $this->writeString();
        //画干扰线
        $this->paitLine();
        //输入图像
        $this->printImg();
    }
    
    /**
     * 创建背景图片
     */
    private function createBackground()
    {
        //从图片库创建一个图像, 判断是否随机
        if ($this->bgRand){
            $img = imagecreatefromjpeg('./verify/bgs/'.mt_rand(1,8).'.jpg');
        }else{
            $img = imagecreatefromjpeg('./verify/bgs/'.$this->type.'.jpg');
        }
        //创建一个图片
        $this->img = imagecreatetruecolor($this->width, $this->height);
        //把图片复制到创建的图像上
        imagecopyresampled($this->img, $img, 0, 0, 0, 0, $this->width, $this->height, imagesx($img), imagesy($img));
    }
    
    /**
     * 在图片上写字
     */
    private function writeString()
    {
        $color = imagecolorallocatealpha($this->img, mt_rand(0,128), mt_rand(0,128), mt_rand(0,128), 0);
        $fontType = './verify/ttfs/'.mt_rand(1,6).'.ttf';
        $fontSize = mt_rand(15, 20);
        for ($i = 0; $i < $this->length; $i++) {
            $x = 3+($this->width/$this->length)*$i;
            $y = mt_rand(($this->height/3)*2, ($this->height/3)*2);
            //把验证码写在图片上
            imagettftext($this->img, $fontSize, 0, $x, $y, $color, $fontType, $this->code[$i]);
        }
    }
    
    /**
     * 画干扰线和字母
     */
    private function paitLine()
    {
        $px = $py = 0;
        $codes = '2345678abcdefhijkmnpqrstuvwxyz';
        for ($i = 0; $i < $this->width/4; $i++){
            $num = mt_rand(0, strlen($codes)-1);
            $color = imagecolorallocatealpha($this->img, 255, 255, 255, 80);
            //画字母
            imagechar($this->img, 8, mt_rand(3, $this->width), mt_rand(3, $this->height), $codes{$num}, $color);
        }
    }
    
    /**
     * 输入图像
     */
    private function printImg()
    {
        if(function_exists('imagegif')){
            // 针对 GIF
            header('Content-Type: image/gif');
            imagegif($this->img);
        }elseif(function_exists('imagejpeg')){
            // 针对 JPEG
            header('Content-Type: image/jpeg');
            imagejpeg($this->img, NULL, 100);
        }elseif(function_exists('imagepng')){
            // 针对 PNG
            header('Content-Type: image/png');
            imagepng($this->img);
        }elseif(function_exists('imagewbmp')){
            // 针对 WBMP
            header('Content-Type: image/vnd.wap.wbmp');
            imagewbmp($this->img);
        }
    }
    
    /**
     * 生成验证码
     * @return string 返回生成的验证码
     */
    private function createVerify()
    {
        $codeSet = '2345678abcdefhijkmnpqrstuvwxyz';
        $codes = '';
        for ($i = 0; $i < $this->length; $i++) {
            $codes .= $codeSet[mt_rand(0, strlen($codeSet)-1)];
        }
        //把验证码保存到session中
        if (!isset($_SESSION)) {session_start();}
        $_SESSION['code'] = $this->encodeVerify(strtolower($codes));
//        $_SESSION['code'] = $codes;
        return $codes;
    }
    
    /**
     * 加密验证码
     * @param type $string
     * @return type
     */
    private function encodeVerify($string)
    {
        $key = substr(md5($this->seKey), 5, 8);
        $str = substr(md5($string), 8, 10);
        return md5($key . $str);
    }
    
    /**
     * 销毁图像
     */
    function __destruct()
    {
        if (isset($this->img)){
            imagedestroy($this->img);
        }
    }
}
